/*
 * softfan.c
 * Main source file for softfan
 * softfan, a parallel port fan controller
 *
 * Copyright (c) 2004 Joshua Wise.
 * Licensed under the GPL. You should've gotten a copy with this distribution.
 * If you didn't, probably because I forgot to include it, I guess you could 
 * write to the FSF and they'd be more than happy to send you a copy. Or you
 * could search Google, which would take a lot less time.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/ppdev.h>

#define FAN0_CONTROL 0

int pfd = -1;

int openport(void)
{
	int err;
	
	pfd = open("/dev/parport0", O_RDWR);
	if (pfd < 0)
	{
		perror("openport: open");
		return pfd;
	}
	
	err = ioctl(pfd, PPEXCL);
	if (err < 0)
	{
		perror("openport: ioctl(PPEXCL)");
		close(pfd);
		pfd = -1;
		return err;
	}
	
	err = ioctl(pfd, PPCLAIM);
	if (err < 0)
	{
		perror("openport: ioctl(PPCLAIM)");
		close(pfd);
		pfd = -1;
		return err;
	}
	
	return pfd;
}

int closeport(void)
{
	int err;
	
	if (pfd < 0)
	{
		fprintf(stderr, "closeport: called without port being open?\n");
		return -1;
	}
	
	err = ioctl(pfd, PPRELEASE);
	if (err < 0)
		perror("closeport: ioctl(PPRELEASE)");
	close(pfd);
	pfd = -1;
	return 0;
}

int setdata(unsigned char d)
{
	int err;
	
	if (pfd < 0)
	{
		fprintf(stderr, "setdata: called without port being open?\n");
		return -1;
	}
	
	err = ioctl(pfd, PPWDATA, &d);
	if (err < 0)
	{
		perror("setdata: ioctl(PPWDATA)");
		return err;
	}
	
	return 0;
}

void usage(char* me)
{
	fprintf(stderr, "softfan 0.10\n");
	fprintf(stderr, "An application to command fans controlled by the parallel port\n"); 
	fprintf(stderr, "\n");
	fprintf(stderr, "Usage: %s mode\n", me);
	fprintf(stderr, "Where mode is one of:\n");
	fprintf(stderr, "  0 for fan full off\n");
	fprintf(stderr, "  1 for fan 50%% on\n");
	fprintf(stderr, "  2 for fan 75%% on\n");
	fprintf(stderr, "  3 for fan full on\n");
	fprintf(stderr, "\n");
	fprintf(stderr, "Report bugs to: Joshua Wise <joshua+softfan@joshuawise.com>\n");
}

int main(int argc, char** argv)
{
	if ((argc != 2) || (argv[1][1] != '\0'))
	{
		usage(argv[0]);
		return 1;
	}
	
	switch(argv[1][0])
	{
	case '0':
		if (openport() < 0)
			return 1;
		if (setdata(0x00))
			return 1;
		while(1)
			sleep(1000000);	// consistent behavior as other modes of operation
		closeport();
		return 0;
	case '1':
		if (openport() < 0)
			return 1;
		while(1)
		{
			if (setdata(1 << FAN0_CONTROL))
				return 1;
			usleep(5000);
			if (setdata(0 << FAN0_CONTROL))
				return 1;
			usleep(5000);
		}
		closeport();
		return 0;
	case '2':
		if (openport() < 0)
			return 1;
		while(1)
		{
			if (setdata(1 << FAN0_CONTROL))
				return 1;
			usleep(7500);
			if (setdata(0 << FAN0_CONTROL))
				return 1;
			usleep(2500);
		}
		closeport();
		return 0;
	case '3':
		if (openport() < 0)
			return 1;
		if (setdata(1 << FAN0_CONTROL))
			return 1;
		while(1)
			sleep(1000000);	// consistent behavior as other modes of operation
		closeport();
		return 0;
	}
	usage(argv[0]);
	return 1;
}
